@@ -1082,13 +1082,6 @@ static int elan_probe(struct i2c_client *client,
 		return error;
 	}
 
-	/* Make sure there is something at this address */
-	error = i2c_smbus_read_byte(client);
-	if (error < 0) {
-		dev_dbg(&client->dev, "nothing at this address: %d\n", error);
-		return -ENXIO;
-	}
-
 	/* Initialize the touchpad. */
 	error = elan_initialize(data);
 	if (error)
@@ -1102,19 +1095,18 @@ static int elan_probe(struct i2c_client *client,
 	if (error)
 		return error;
 
+	dev_info(&client->dev,
+		 "Elan Touchpad: Module ID: 0x%04x, Firmware: 0x%04x, Sample: 0x%04x, IAP: 0x%04x\n",
+		 data->product_id,
+		 data->fw_version,
+		 data->sm_version,
+		 data->iap_version);
+
 	dev_dbg(&client->dev,
-		"Elan Touchpad Information:\n"
-		"    Module product ID:  0x%04x\n"
-		"    Firmware Version:  0x%04x\n"
-		"    Sample Version:  0x%04x\n"
-		"    IAP Version:  0x%04x\n"
+		"Elan Touchpad Extra Information:\n"
 		"    Max ABS X,Y:   %d,%d\n"
 		"    Width X,Y:   %d,%d\n"
 		"    Resolution X,Y:   %d,%d (dots/mm)\n",
-		data->product_id,
-		data->fw_version,
-		data->sm_version,
-		data->iap_version,
 		data->max_x, data->max_y,
 		data->width_x, data->width_y,
 		data->x_res, data->y_res);
@@ -1241,15 +1233,7 @@ static const struct acpi_device_id elan_acpi_id[] = {
 	{ "ELAN0000", 0 },
 	{ "ELAN0100", 0 },
 	{ "ELAN0600", 0 },
-	{ "ELAN0602", 0 },
 	{ "ELAN0605", 0 },
-	{ "ELAN0608", 0 },
-	{ "ELAN0605", 0 },
-	{ "ELAN0609", 0 },
-	{ "ELAN060B", 0 },
-	{ "ELAN060C", 0 },
-	{ "ELAN0611", 0 },
-	{ "ELAN0612", 0 },
 	{ "ELAN1000", 0 },
 	{ }
 };
